/***************************************************************************
 ** 
 **  File: readbinex1.c
 **
 **  Description: An example of using the routines in readbin.c.
 **
 **  Date: 8th January 2004
 **
 ***************************************************************************
 ***************************************************************************
 *
 *    Copyright (C) 2004 Scott A. Belmonte
 *    All rights reserved.
 *
 *    Redistribution and use in source and binary forms, with or without 
 *    modification, are permitted provided that the following conditions
 *    are met:
 *
 *    Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 *    Neither the name of the copyright holder nor the names of any
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 *    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 *    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 *    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "readbin.h"

#define TIFF_BIG_ENDIAN 0x4D4D
#define TIFF_LITTLE_ENDIAN 0x4949
#define TIFF_VERSION 42

typedef struct
{
  unsigned short byte_order;  /* Little-endian or big-endian. */
  unsigned short version;     /* Tiff version number. */
  unsigned int   first_ifd_offset;
} t_tiff_header;

int main()
{
    FILE          *file_p;
    t_tiff_header  tiff_header;
    e_swap         swap_bytes;

    /* Open example.tif */
    if ((file_p = fopen("example.tif", "rb")) == NULL)
    {
        fprintf(stderr, "Error opening file.\n");
        exit(1);
    }

    /* Read byte order marker. */
    if (read_bin_file(&tiff_header.byte_order, sizeof(unsigned short),
                      1, file_p, DONT_SWAP) != 1)
    {
        fprintf(stderr, "Error reading byte order.\n");
        exit(1);
    }

    /* Check byte order marker is valid */
    if (tiff_header.byte_order != TIFF_BIG_ENDIAN &&
        tiff_header.byte_order != TIFF_LITTLE_ENDIAN)
    {
        fprintf(stderr, "Error reading TIFF header: Illegal byte order %hX\n",
	            tiff_header.byte_order);
        exit(1);
    }
  
    /* Work out if we need to swap multibyte sequences */
    if ((tiff_header.byte_order == TIFF_BIG_ENDIAN &&
        get_endian() == MACHINE_LITTLE_ENDIAN) ||
        (tiff_header.byte_order == TIFF_LITTLE_ENDIAN &&
        get_endian() == MACHINE_BIG_ENDIAN))
    {
        swap_bytes = SWAP;
    }
    else
    {
        swap_bytes = DONT_SWAP;
    }
    
    /* Read TIFF version. */
    if (read_bin_file(&tiff_header.version, sizeof(unsigned short),
                      1, file_p, swap_bytes) != 1)
    {
        fprintf(stderr, "Error reading version.\n");
        exit(1);
    }

    /* Read TIFF version. */
    if (read_bin_file(&tiff_header.first_ifd_offset, sizeof(unsigned int),
                      1, file_p, swap_bytes) != 1)
    {
        fprintf(stderr, "Error reading version.\n");
        exit(1);
    }

	fclose(file_p);
    return 0;
}
